/*********************************************************/
/*  Program testfft2.c                                    */
/*     An example of how to use function fft.c           */
/*********************************************************/

#include "complex.h"

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

   extern void fft(struct cmpx *XX, int MM);

  #define M  6  /* M = log2(N) = number of FFT stages */
  #define N 64 /* N = 2**M = number of FFT points */
  #define Nsine 48 /* Size of sine table = 3*N/4 */

  struct cmpx X[N];
  float sinetable[Nsine];
  float fftmag[N];

main()
  {
  int i;                   /* loop index                 */
  double pi = 3.141592653589;
  double twopi = 2*pi;
  double twopi_over_N = twopi/N;
  
//  FILE *fp, *fopen();
 
/*-------------------------------------------------------*/
/*   Initialize input array                              */
/*  Generate spectral lines at k = 5 and 11 of height 8. */

  for(i=0; i<N; i++) 
    {
    X[i].real = cos(i*5*2.0*pi/N);
    X[i].imag = 0.0;
    }
/*--------------------------------------------------------*/

/*--------------------------------------------------------*/
/*  Generate the sine table                               */
/*   Note: cos x = sin (x + pi/2). Therefore cos x can be */
/* obtained from the table as cos[n] = sinetable[n + N/4].*/
/* The FFT uses sine values for N in the range [0, N/2).  */
/* The array size is defined to be Nsine = 3N/4 to easily */
/* address the cosine value.                              */ 

   for(i=0; i < Nsine; i++)
   {
   sinetable[i] = (float) sin(i*twopi_over_N);
   }

/* Perform FFT                                            */

  fft(X,M);
  
   for(i=0; i<N; i++)
      fftmag[i] = (float)(X[i].real*X[i].real + X[i].imag*X[i].imag); 

/************************************************************/
/*  Write out FFT to console                                */
/************************************************************/
  for(i=0; i<N; i++)
    printf("%4d%15.5f\t%15.5f\n",i,X[i].real, X[i].imag);
  
/**************************************************************/
/* Write FFT to file junk.dat                                 */
/**************************************************************/

/*      
   fp = fopen("junk.dat", "w");
  for(i=0; i<N; i++)
    fprintf(fp,"%4d%15.5f\t%15.5f\n",i,X[i].real, X[i].imag);
  fclose(fp);
*/

}
