C  SUBROUTINE TO GENERATE LEAST SQUARES POLYNOMIAL FIT
C
        SUBROUTINE POLYFIT(NUM,XX,YY,IDEGRE,ODEGRE,COEFS)
C
C  INPUTS
C
C       NUM     NUMBER OF DATA POINTS
C       XX      X COORDINATES OF INPUT POINTS (DOUBLE PRECISION)
C       YY      Y COORDINATES OF INPUT POINTS (DOUBLE PRECISION)
C       IDEGRE  DESIRED DEGREE OF POLYNOMIAL FIT
C
C  OUTPUTS
C
C       ODEGRE  ACTUAL DEGREE OF COMPUTED FIT
C       COEFS   COEFFICIENTS OF FIT. THIS IS A DOUBLE PRECISION ARRAY
C                 WITH DIMENSION 23. THUS DEFINE COEFS IN MAIN PROGRAM
C                 AS:  DOUBLE PRECISION COEFS(23)
C
        INTEGER ODEGRE
        DOUBLE PRECISION WORK,DD,SUMSQS,AA,BB,COEFS,XX,YY
        DIMENSION XX(NUM), YY(NUM), COEFS(23), SUMSQS(23)
        DIMENSION AA(23), BB(23), WORK(5020,2),DD(4,21)
        CONTRL=100.
        IF(NUM.GT.5000) THEN
            WRITE(*,*) ' NUMBER OF POINTS MUST BE LESS THAN 5000.'
            STOP
        ENDIF
        IF(IDEGRE.GT.20) THEN
           WRITE(*,*) ' '
           WRITE(*,*) ' DEGREE MUST NOT EXCEED 20'
           STOP
        ENDIF
        IF(IDEGRE.GE.NUM) THEN
           WRITE(*,*) ' '
           WRITE(*,*) 'DEGREE.GE.NUMBER OF POINTS. MUST BE SMALLER.'
           STOP
        ENDIF
      CALL FIT1(XX,YY,NUM,CONTRL,IDEGRE,ODEGRE,WORK,COEFS,SUMSQS,AA,BB)
      CALL FIT2(COEFS,ODEGRE,AA,BB,DD)
       RETURN
       END
C******************************************************************
        SUBROUTINE  FIT1(XX,YY,NUM,CONTRL,IDEGRE,ODEGRE,WORK,COEFS,
     & SUMSQS,AA,BB)
C
C    INPUTS
C
C  XX           X ARRAY
C  YY           Y ARRAY
C  NUM          NUMBER OF POINTS IN X AND Y ARRAYS
C  CONTRL       ERROR TOLLERANCE
C  IDEGRE       DESIRED POLYNOMIAL DEGREE
C
C    OUTPUTS
C
C  ODEGRE       HIGHEST DEGREE COMPUTED
C  WORK         ORTHOGONALIZED POLYNOMIALS P(N) AND P(N-1)
C  COEFS        BEST FIT POLYNOMIAL COEFFICIENTS
C  SUMSQS
C  AA
C  BB
C
        INTEGER ODEGRE,ODEG1,ERROR
        DOUBLE PRECISION X1,X3,Y1,Y2,WORK,RR,SS,TT,UU,VV
        DOUBLE PRECISION SUMSQS,AA,BB,WW,ZZ,X2,X4
        DOUBLE PRECISION COEFS,XX,YY
        DIMENSION XX(NUM), YY(NUM), COEFS(23), SUMSQS(23)
        DIMENSION AA(23), BB(23), WORK(NUM,2)
        IREAD=4
        IRITE=4
        ERROR=0
        RR=0.D0
        IF (IDEGRE.LT.NUM.AND.IDEGRE.GE.1)GO TO 5
        ERROR=1
        GO TO 90
  5     IF (CONTRL.GT.0.0)GO TO 10
        ERROR=3
        GO TO 90
 10     IF (CONTRL.LE.100.0)GO TO 15
        ERROR=4
        CONTRL=100.0
 15     IF (IDEGRE.GT.20)ERROR=5
        X4=XX(1)
        X2=XX(1)
        NN=1
        DO 20 JJ=2,NUM
        IF (YY(JJ-1).EQ.YY(JJ))NN=NN+1
        IF (XX(JJ).GT.X4)X4=XX(JJ)
        IF (XX(JJ).LT.X2)X2=XX(JJ)
 20     CONTINUE
        X1=X2
        X3=X4
        IF (X1.NE.X3.AND.NN.NE.NUM)GO TO 25
        ERROR=2
        GO TO 90
 25     Y1=(X1+X3)*.5D0
        Y2=4.D0/(X3-X1)
        I1=1
        I2=2
        D=0.D0
        DO 30 JJ=1,NUM
        WORK(JJ,I1)=1.D0
        SS=Y2*(XX(JJ)-Y1)
        RR=RR+SS
        XX(JJ)=SS
 30     CONTINUE
        RR=RR/NUM
        AA(1)=RR
        BB(1)=0.0
        TT=0.D0
        DO 35 JJ=1,NUM
        TT=TT+YY(JJ)
        WORK(JJ,I2)=XX(JJ)-RR
 35     CONTINUE
        X3=0.D0
        DO 40 JJ=1,NUM
        X3=X3 + WORK(JJ,I2) * WORK(JJ,I2)
 40     CONTINUE
        UU=0.D0
        DO 45 JJ=1,NUM
        UU=UU + DBLE(YY(JJ)) * DBLE(YY(JJ))
 45     CONTINUE
        SUMSQS(1)=UU
        WW=CONTRL*.01
        ZZ=0.0
        SUMSQS(2)=TT*TT/NUM
        UU=UU-SUMSQS(2)
        RR=NUM
        COEFS(1)=TT/NUM
        IDEG1=IDEGRE+1
        ODEGRE=1
        TT=0.D0
        DO 50 JJ=1,NUM
        TT=TT + WORK(JJ,I2) * YY(JJ)
 50     CONTINUE
        COEFS(2)=TT/X3
        IF (IDEG1.LT.2)GO TO 80
        DO 75 JJ=2,IDEG1
        KK=JJ+1
        SUMSQS(KK)=TT*TT/X3
        ZZ=ZZ+SUMSQS(KK)
        UU=UU-SUMSQS(KK)
        IF (UU.LE.0.D0)UU=0.D0
        IF (ZZ/(SUMSQS(1)-SUMSQS(2)).GT.WW) GO TO 80
        IF (JJ.EQ.IDEG1)GO TO 80
        ODEGRE=JJ
        VV=0.D0
        DO 55 LL=1,NUM
        VV=VV+XX(LL)*WORK(LL,I2)*WORK(LL,I2)
 55     CONTINUE
        VV=VV/X3
        X1=X3/RR
        DO 60 LL=1,NUM
        WORK(LL,I1)=(XX(LL)-VV)*WORK(LL,I2)-X1*WORK(LL,I1)
 60     CONTINUE
        AA(JJ)=VV
        BB(JJ)=X1
        RR=X3
        TT=0.D0
        DO 65 LL=1,NUM
        TT=TT + WORK(LL,I1) * YY(LL)
 65     CONTINUE
        X3=0.D0
        DO 70 KL=1,NUM
        X3=X3 + WORK(KL,I1) * WORK(KL,I1)
 70     CONTINUE
        COEFS(KK)=TT/X3
        II=I1
        I1=I2
        I2=II
 75     CONTINUE
 80     SUMSQS(IDEGRE+3)=UU
        COEFS(ODEGRE+2)=Y2
        COEFS(ODEGRE+3)=-Y2*Y1
        IF (ERROR)90,999,90
 90     CONTINUE
        WRITE(IRITE,500)
500     FORMAT (//' *******   ERROR MESSAGE   *******' )
        IF (ERROR.EQ.1)WRITE(IRITE,505)
505     FORMAT (/' NOT ENOUGH DATA POINTS WERE GIVEN FOR THE DESIRED' ,
     1 ' DEGREE,' ,/' OR, A ZERO ORDER POLYNOMIAL FIT WAS DESIRED.' )
        IF (ERROR.EQ.2) WRITE(IRITE,510)
510     FORMAT (/' THE X VECTOR, OR THE Y VECTOR, WAS CONSTANT.' )
        IF (ERROR.EQ.3) WRITE(IRITE,515)
515     FORMAT (/' THE "CONTRL" VARIABLE WAS .LE. 0.0' )
        IF (ERROR.EQ.4) WRITE(IRITE,520)
520     FORMAT (/' THE "CONTRL" VARIABLE WAS .GT. 100.0' )
        IF (ERROR.EQ.5) WRITE(IRITE,525)
525     FORMAT (/' THE DESIRED DEGREE OF FIT IS GREATER THAN 20' )
 999    RETURN
        END
C
C********************************************************************
        SUBROUTINE FIT2(COEFS,ODEGRE,AA,BB,DD)
        INTEGER ODEGRE,ODEG1
        DIMENSION COEFS(23),AA(23),BB(23),DD(4,21)
        REAL*8 ALPHA,BETA,DD,EE,FF,GG,PP,QQ,AA,BB
        DOUBLE PRECISION COEFS,CC
        ODEG1=ODEGRE+1
        DO 5 JJ=1,ODEG1
        DD(4,JJ)=COEFS(JJ)
        DO 5 LL=1,3
 5      DD(LL,JJ)=0.0D0
        DO 15 JJ=2,ODEG1
        DD(2,JJ)=1.0D0
        LL=JJ-1
        ALPHA=AA(LL)
        BETA=BB(LL)
        DO 10  II=2,JJ
        MM=II-1
        DD(3,II)=DD(2,MM)-DD(2,II)*ALPHA-BETA*DD(1,II)
 10     DD(4,MM)=DD(4,MM)+COEFS(JJ)*DD(3,II)
        IF (JJ.EQ.ODEG1)GOTO 20
        DO 15 II=1,JJ
        DD(1,II)=DD(2,II)
 15     DD(2,II)=DD(3,II)
 20     BETA=COEFS(ODEGRE+2)
        FF=1.0D0
        GG=COEFS(ODEGRE+3)
        CC=1.0D0
        QQ=1.0D0
        PP=1.0D0
        DO 40 KK=1,ODEG1
        EE=FF*DD(4,KK)
        LL=KK+1
        IF (LL.GT.ODEG1)GO TO 35
        IF (KK.EQ.1)GO TO 25
        PP=KK
        QQ=1.D0
 25     DO 30 JJ=LL,ODEG1
        CC=GG*CC*PP/QQ
        EE=EE+DD(4,JJ)*CC
        IF (KK.EQ.1)GOTO 30
        PP=PP+1.D0
        QQ=QQ+1.D0
 30     CONTINUE
 35     COEFS(KK)=EE
        FF=FF*BETA
 40     CC=FF
        RETURN
        END
C******************************************************************
